"""Main module."""
import logging
from typing import List
import os
import pandas as pd
import xarray as xr
from shapely.geometry import Point
import geopandas as gpd
import numpy as np
from osgeo import gdal
from osgeo import gdalconst

import sys
if not sys.warnoptions:
    import warnings
    warnings.simplefilter("ignore")

logger = logging.getLogger(__name__)

def netcdf2df(
    netcdf: str
) -> pd.DataFrame:
    """
    Produce a dataframe from a NetCDF4 file.

    Parameters
    ----------
    netcdf: str
        Path to the netcdf file

    Returns
    -------
    DataFrame
        The resultant dataframe
    """
    try:
        ds = xr.open_dataset(netcdf)
    except:
        raise AssertionError(f"improperly formatted netCDF file ({netcdf})")

    data = ds.to_dataframe()
    df = data.reset_index()
    return df


def raster2df(
    InRaster: str, feature_name: str='feature', band: int=1, nodataval: int=-9999
) -> pd.DataFrame:
    '''
    Description
    -----------
    Takes the path of a raster (.tiff) file and produces a Geopandas Data Frame.
    
    Parameters
    ----------
    InRaster: str
        the path of the input raster file
    feature_name: str
        the name of the feature represented by the pixel values 
    band: int, default 1
        the band to operate on
    nodataval: int, default -9999
        the value for no data pixels

    Examples
    --------
    Converting a geotiff of rainfall data into a geopandas dataframe

    >>> df = raster2df('path_to_raster.geotiff', 'rainfall', band=1)

    '''

    # open the raster and get some properties
    ds       = gdal.OpenShared(InRaster,gdalconst.GA_ReadOnly)
    GeoTrans = ds.GetGeoTransform()
    ColRange = range(ds.RasterXSize)
    RowRange = range(ds.RasterYSize)
    rBand    = ds.GetRasterBand(band) # first band
    nData    = rBand.GetNoDataValue()
    if nData == None:
        logging.info(f"No nodataval found, setting to {nodataval}")
        nData = np.float32(nodataval) # set it to something if not set
    else:
        logging.info(f"Nodataval is: {nData}")

    # specify the center offset (takes the point in middle of pixel)
    HalfX    = GeoTrans[1] / 2
    HalfY    = GeoTrans[5] / 2

    # Check that NoDataValue is of the same type as the raster data
    RowData = rBand.ReadAsArray(0,0,ds.RasterXSize,1)[0]
    if type(nData) != type(RowData[0]):
        logging.warning(f"NoData type mismatch: NoDataValue is type {type(nData)} and raster data is type {type(RowData[0])}")
        

    points = []
    for ThisRow in RowRange:
        RowData = rBand.ReadAsArray(0,ThisRow,ds.RasterXSize,1)[0]
        for ThisCol in ColRange:
            # need to exclude NaN values since there is no nodataval
            if (RowData[ThisCol] != nData) and not (np.isnan(RowData[ThisCol])):
                
                # TODO: implement filters on valid pixels
                # for example, the below would ensure pixel values are between -100 and 100
                #if (RowData[ThisCol] <= 100) and (RowData[ThisCol] >= -100):

                X = GeoTrans[0] + ( ThisCol * GeoTrans[1] )
                Y = GeoTrans[3] + ( ThisRow * GeoTrans[5] ) # Y is negative so it's a minus
                # this gives the upper left of the cell, offset by half a cell to get centre
                X += HalfX
                Y += HalfY

                points.append([X,Y,RowData[ThisCol]])

    return pd.DataFrame(points, columns=['longitude','latitude',feature_name])


def geocode(
    df: pd.DataFrame, x: str='longitude', y: str='latitude'
) -> pd.DataFrame:
    '''
    Description
    -----------
    Takes a dataframe containing coordinate data and geocodes it to GADM (https://gadm.org/)
    
    Parameters
    ----------
    df: pd.DataFrame
        a pandas dataframe containing point data
    x: str, default 'longitude'
        the name of the column containing longitude information
    y: str, default 'latitude'
        the name of the column containing latitude data

    Examples
    --------
    Geocoding a dataframe with columns named 'lat' and 'lon'

    >>> df = geocode(df, x='lon', y='lat')

    '''
    gadm_data_dir = os.path.expanduser("~") 
    gadm = gpd.read_file(f"{gadm_data_dir}/mixmasta_data/gadm/gadm36_2.shp")
    gadm['country'] = gadm['NAME_0']
    gadm['state'] = gadm['NAME_1']
    gadm['admin1'] = gadm['NAME_1']
    gadm['admin2'] = gadm['NAME_2']
    gadm = gadm[['geometry','country','state','admin1','admin2']]

    df['geometry'] = df.apply(lambda row: Point(row[x], row[y]), axis=1)
    gdf = gpd.GeoDataFrame(df)

    # Spatial merge on GADM to obtain admin areas
    gdf = gpd.sjoin(gdf, gadm, how="left", op='intersects')
    del(gdf['geometry'])
    del(gdf['index_right'])
    return pd.DataFrame(gdf)


"""Console script for mixmasta."""
import sys
import click
from .download import download_and_clean


@click.command()
@click.argument('command')
def main(command):
    """Console script for mixmasta."""
    if command == 'download':
    	download_and_clean()
    else:
        click.echo("Replace this message by putting your code into "
                   "mixmasta.cli.main")
        click.echo("See click documentation at https://click.palletsprojects.com/")
    return 0

if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

def read_requirements(path: str):
    with open(path) as f:
        return f.read().splitlines()

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.md') as history_file:
    history = history_file.read()

install_requirements = read_requirements("requirements.txt")

setup(
    author="Brandon Rose",
    author_email='brandon@jataware.com',
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="A library for common scientific model transforms",
    entry_points={
        'console_scripts': [
            'mixmasta=mixmasta.cli:main',
        ],
    },
    setup_requires=['numpy>=1.20.1'], # This is not working as expected
    install_requires=install_requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/markdown',
    include_package_data=True,
    keywords='mixmasta',
    name='mixmasta',
    packages=find_packages(include=['mixmasta', 'mixmasta.*']),
    test_suite='tests',
    url='https://github.com/jataware/mixmasta',
    version='0.2.4',
    zip_safe=False,
)



import requests
import shutil
import zipfile
import os
import sys
from urllib.request import urlretrieve
from tqdm import tqdm

# This is used to show progress when downloading.
# see here: https://github.com/tqdm/tqdm#hooks-and-callbacks
class TqdmUpTo(tqdm):
    """Provides `update_to(n)` which uses `tqdm.update(delta_n)`."""
    def update_to(self, b=1, bsize=1, tsize=None):
        """
        b  : int, optional
            Number of blocks transferred so far [default: 1].
        bsize  : int, optional
            Size of each block (in tqdm units) [default: 1].
        tsize  : int, optional
            Total size (in tqdm units). If [default: None] remains unchanged.
        """
        if tsize is not None:
            self.total = tsize
        self.update(b * bsize - self.n)  # will also set self.n = b * bsize

def download_file(url, fname):
    with requests.get(url, stream=True) as r:
        with open(fname, 'wb') as f:
            shutil.copyfileobj(r.raw, f)
    return fname

def download_progress(url, fname):
    """Download a file and show a progress bar."""
    with TqdmUpTo(unit='B', unit_scale=True, miniters=1,
              desc=url.split('/')[-1]) as t:  # all optional kwargs
        urlretrieve(url, filename=fname, reporthook=t.update_to, data=None)
        t.total = t.n
    return fname

def download_and_clean(version=1, 
                       url="https://jataware-world-modelers.s3.amazonaws.com/mixmasta.zip", 
                       dirname='mixmasta_data'):
    """Download mixmasta and prep the GADM directory.
    This downloads the zip file from the source, extracts it, renames the
    resulting directory, and removes large files not used at runtime.  
    """
    cdir = os.path.expanduser("~")
    print(cdir)
    fname = os.path.join(cdir, 'mixmasta.zip')
    print("Downloading mixmasta v{}...".format(version), file=sys.stderr)
    download_progress(url, fname)
    print("Finished download.")

    outdir = os.path.join(cdir, dirname)
    with zipfile.ZipFile(fname, 'r') as zf:
        zf.extractall(outdir)
    os.rename(f"{outdir}/data", f"{outdir}/gadm")
    os.remove(fname)

    # save a version file so we can tell what it is
    vpath = os.path.join(outdir, 'version')
    with open(vpath, 'w') as vfile:
        vfile.write('mixmasta-{}'.format(version))

    print("Downloaded mixmasta v{} to {}".format(version, outdir), file=sys.stderr)